//////////////////////////////////////////////////////////////////////
//! \file wldv32.h
//! \author Schober
//! \brief  defines for interfacing to the dll
//! \note   this is the header file for the uses
//
//////////////////////////////////////////////////////////////////////

#ifndef WLDV32_H
#define WLDV32_H		                //!< prevents C++ header nesting

#ifndef _WINDOWS_
  #error Please include first <windows.h> or <afxwin.h>
#endif


#ifndef LPC_DLL_H
//! \typedef LDVCode
//! \brief  Network driver return codes 
typedef enum {
	LDV_OK				= 0,    		//!< everything oK
	LDV_NOT_FOUND		= 1,			//!< unused, left for compatibility
	LDV_ALREADY_OPEN	= 2,            //!< WLDV32 has already been opened
	LDV_NOT_OPEN		= 3,    	    //!< WLDV32 must have been opened for this call
	LDV_DEVICE_ERROR	= 4,            //!< error during write to device
	LDV_NO_MSG_AVAIL	= 6,            //!< no data ready from interface
	LDV_NO_BUFF_AVAIL	= 7,            //!< no message buffer to store neuron id
	LDV_NO_RESOURCES	= 8,		    //!< unused, left for compatibility
	LDV_OPEN_ERROR		= -1,           //!< invalid device name
	LDV_NO_LICENSE		= -2,           //!<  WLDV has not been registered for this device
	LDV_NOT_CONNECTED	= -3,           //!<  Communication with the device failed
	//new codes in multi-client version
	LDV_NO_DRIVER		= -4,   	    //!< LON driver not found
	LDV_COMERROR		= -5,   	    //!< communication error
	LDV_NO_THREAD		= -6,   	    //!< no listening thread
	LDV_HANDLE_INVALID  = -7,           //!< wrong handle passed by client
	LDV_OTHERDEVICE		= -8,           //!< open device uses another driver
	LDV_NOPIPE			= -9,   	    //!< pipe server/client not created
	LDV_READPIPE		= -10,  	    //!< pipe read / write error
	LDV_READNEURONID	= -11           //!< error reading licence data
} LDVCode;

//! \typedef LNI
//! \brief LNI is used as handle for an interface
typedef int LNI;


// modes for ldv_debugmode()
#define WLDV32_DEBUG_FILE      0x0001
#define WLDV32_DEBUG_DBGVIEW   0x0002
#define WLDV32_DEBUG_SERIAL    0x0004
#define WLDV32_DEBUG_REPLY     0x0100


#if !defined(RC_INCLUDE)

#if !defined(_DLLEXPORT_)

// If _DLLEXPORT_ is not defined then the default is to import.
#if defined(__cplusplus)
#define DLLENTRY extern "C" __declspec(dllimport)						//!< standard Windows define for a DLL function
#define STDENTRY extern "C" __declspec(dllimport) HRESULT WINAPI		//!< standard Windows define for a DLL function with a HRESULT as return type
#define STDENTRY_(type) extern "C" __declspec(dllimport) type WINAPI 	//!< standard Windows define for a DLL function with "type" as return type
#else
#define DLLENTRY __declspec(dllimport)									//!< standard Windows define for a DLL function
#define STDENTRY __declspec(dllimport) HRESULT WINAPI				    //!< standard Windows define for a DLL function with a HRESULT as return type
#define STDENTRY_(type) __declspec(dllimport) type WINAPI		        //!< standard Windows define for a DLL function with "type" as return type
#endif

#else  // _DLLEXPORT_
// Else if _DLLEXPORT_ is defined then we've been told to export.

#if defined(__cplusplus)
#define DLLENTRY extern "C" __declspec(dllexport)						//!< standard Windows define for a DLL function
#define STDENTRY extern "C" __declspec(dllexport) HRESULT WINAPI		//!< standard Windows define for a DLL function with a HRESULT as return type
#define STDENTRY_(type) extern "C" __declspec(dllexport) type WINAPI	//!< standard Windows define for a DLL function with "type" as return type
#else
#define DLLENTRY __declspec(dllexport)									//!< standard Windows define for a DLL function
#define STDENTRY __declspec(dllexport) HRESULT WINAPI					//!< standard Windows define for a DLL function with a HRESULT as return type
#define STDENTRY_(type) __declspec(dllexport) type WINAPI			    //!< standard Windows define for a DLL function with "type" as return type
#endif

#endif // _DLLEXPORT_

// Here is the list of service APIs offered by the DLL (using the
//   appropriate entry API declaration macros just #defined above).
STDENTRY_(char*)   ldv_get_version(void);
STDENTRY_(LNI)     ldv_open(const char* device_name);
STDENTRY_(LDVCode) ldv_close(LNI handle);
STDENTRY_(LDVCode) ldv_read(LNI handle, void* msg_p, unsigned length);
STDENTRY_(LDVCode) ldv_write(LNI handle, const void* msg_p, unsigned length);
STDENTRY_(LDVCode) ldv_register(LNI handle, void (*func_p)(void));
STDENTRY_(LDVCode) ldv_debugmode(char* name, int mode);
STDENTRY_(LDVCode) ldv_neuronID(void* msg_p);

STDENTRY_(LNI)     ldv_openex(const char* device_name);
STDENTRY_(char*)   ldv_get_versionex(LNI handle);
STDENTRY_(LDVCode) ldv_neuronIDex(LNI handle, void* msg_p);
STDENTRY_(LDVCode) ldv_registerevent(LNI handle, HANDLE hEvent);

#endif // RC_INCLUDE

#endif  // LPC_DLL_H


// Function pointer types for dynamic dll loading
typedef char*   (WINAPI *WLDV_GET_VERSION)(void);		                        //!< type for ldv_get_version for dynamic dll loading
typedef LNI     (WINAPI *WLDV_OPEN)(char* device_name);	                        //!< type for ldv_open for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_CLOSE)(LNI handle);	                            //!< type for ldv_close for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_READ)(LNI handle, void* msg_p, unsigned length);	//!< type for ldv_read for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_WRITE)(LNI handle, const void* msg_p, unsigned length);	//!< type for ldv_write for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_REGISTER)(LNI handle, void (*func_p)(void));	    //!< type for ldv_register for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_DEBUGMODE)(char* name, int mode);	                //!< type for ldv_debugmode for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_NEURONID)(void* msg_p);	                        //!< type for ldv_neuronID for dynamic dll loading

typedef LNI     (WINAPI *WLDV_OPENEX)(char* device_name);	                    //!< type for ldv_openex for dynamic dll loading
typedef char*   (WINAPI *WLDV_GET_VERSIONEX)(LNI handle);	                    //!< type for ldv_get_versionex for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_NEURONIDEX)(LNI handle, void* msg_p);	            //!< type for ldv_neuronIDex for dynamic dll loading
typedef LDVCode (WINAPI *WLDV_REGISTEREVENT)(LNI handle, HANDLE hEvent);	    //!< type for ldv_registerevent for dynamic dll loading


#endif // WLDV32_H
